--- wine-20050419/dlls/iphlpapi/ifenum.c.20050419	2006-01-16 17:33:15.000000000 -0800
+++ wine-20050419/dlls/iphlpapi/ifenum.c	2006-01-24 15:38:16.000000000 -0800
@@ -184,40 +184,6 @@
   return map;
 }
 
-static int isLoopbackInterface(int fd, const char *name)
-{
-  int ret = 0;
-
-  if (name) {
-    struct ifreq ifr;
-
-    lstrcpynA(ifr.ifr_name, name, IFNAMSIZ);
-    if (ioctl(fd, SIOCGIFFLAGS, &ifr) == 0)
-      ret = ifr.ifr_flags & IFF_LOOPBACK;
-  }
-  return ret;
-}
-
-static void countInterfaces(int fd, caddr_t buf, size_t len)
-{
-  caddr_t ifPtr = buf;
-  DWORD numNonLoopbackInterfaces = 0, numLoopbackInterfaces = 0;
-
-  while (ifPtr && ifPtr < buf + len) {
-    struct ifreq *ifr = (struct ifreq *)ifPtr;
-
-    if (isLoopbackInterface(fd, ifr->ifr_name))
-      numLoopbackInterfaces++;
-    else
-      numNonLoopbackInterfaces++;
-    ifPtr += ifreq_len(ifr);
-  }
-  gNonLoopbackInterfaceMap = sizeMap(gNonLoopbackInterfaceMap,
-   numNonLoopbackInterfaces);
-  gLoopbackInterfaceMap = sizeMap(gLoopbackInterfaceMap,
-   numLoopbackInterfaces);
-}
-
 /* Stores the name in the given map, and increments the map's numInterfaces
  * member if stored successfully.  Will store in the same slot as previously if
  * usedLastPass is set, otherwise will store in a new slot.
@@ -272,6 +238,105 @@
   }
 }
 
+#ifdef HAVE_NET_IF_H
+static int isLoopbackInterface(int fd, const char *name)
+{
+  int ret = 0;
+
+  if (name) {
+    struct ifreq ifr;
+
+    lstrcpynA(ifr.ifr_name, name, IFNAMSIZ);
+    if (ioctl(fd, SIOCGIFFLAGS, &ifr) == 0)
+      ret = ifr.ifr_flags & IFF_LOOPBACK;
+  }
+  return ret;
+}
+
+static void enumerateInterfaces(void)
+{
+  int fd;
+  int nonLoopbacks;
+  int loopbacks;
+  int i;
+  char buffer[IF_NAMESIZE];
+  const char* name;
+
+  fd = socket(PF_INET, SOCK_DGRAM, 0);
+  if (fd != -1) {
+    EnterCriticalSection(&mapCS);
+
+    nonLoopbacks = 0;
+    loopbacks = 0;
+    #define MOST_INTERFACES_IMAGINEABLE 50
+    for (i = 0; i < MOST_INTERFACES_IMAGINEABLE; ++i) {
+      name = if_indextoname(i, buffer);
+      if (name) {
+        if (isLoopbackInterface(fd, name))
+          loopbacks++;
+        else
+          nonLoopbacks++;
+      }
+    }
+    gNonLoopbackInterfaceMap = sizeMap(gNonLoopbackInterfaceMap,
+     nonLoopbacks);
+    gLoopbackInterfaceMap = sizeMap(gLoopbackInterfaceMap,
+     loopbacks);
+
+    markOldInterfaces(gNonLoopbackInterfaceMap);
+    markOldInterfaces(gLoopbackInterfaceMap);
+
+    /* add table entries for them them */
+    for (i = 0; i < MOST_INTERFACES_IMAGINEABLE; ++i) {
+      name = if_indextoname(i, buffer);
+      if (name) {
+        if (isLoopbackInterface(fd, name))
+          storeInterfaceInMap(gLoopbackInterfaceMap, name);
+        else
+          storeInterfaceInMap(gNonLoopbackInterfaceMap, name);
+      }
+    }
+
+    LeaveCriticalSection(&mapCS);
+  }
+  else
+    perror("iphlpapi/ifenum.c::socket()");
+}
+#else	 // !HAVE_NET_IF_H
+static int isLoopbackInterface(int fd, const char *name)
+{
+  int ret = 0;
+
+  if (name) {
+    struct ifreq ifr;
+
+    lstrcpynA(ifr.ifr_name, name, IFNAMSIZ);
+    if (ioctl(fd, SIOCGIFFLAGS, &ifr) == 0)
+      ret = ifr.ifr_flags & IFF_LOOPBACK;
+  }
+  return ret;
+}
+
+static void countInterfaces(int fd, caddr_t buf, size_t len)
+{
+  caddr_t ifPtr = buf;
+  DWORD numNonLoopbackInterfaces = 0, numLoopbackInterfaces = 0;
+
+  while (ifPtr && ifPtr < buf + len) {
+    struct ifreq *ifr = (struct ifreq *)ifPtr;
+
+    if (isLoopbackInterface(fd, ifr->ifr_name))
+      numLoopbackInterfaces++;
+    else
+      numNonLoopbackInterfaces++;
+    ifPtr += ifreq_len(ifr);
+  }
+  gNonLoopbackInterfaceMap = sizeMap(gNonLoopbackInterfaceMap,
+   numNonLoopbackInterfaces);
+  gLoopbackInterfaceMap = sizeMap(gLoopbackInterfaceMap,
+   numLoopbackInterfaces);
+}
+
 static void classifyInterfaces(int fd, caddr_t buf, size_t len)
 {
   caddr_t ifPtr = buf;
@@ -335,6 +400,7 @@
     close(fd);
   }
 }
+#endif
 
 DWORD getNumNonLoopbackInterfaces(void)
 {
